package java.text;

import sun.util.locale.provider.LocaleServiceProviderPool;
import sun.util.locale.provider.LocaleProviderAdapter;
import java.text.spi.BreakIteratorProvider;
import java.util.Locale;
import java.lang.ref.SoftReference;

public abstract class BreakIterator implements Cloneable
{
    public static final int DONE = -1;
    private static final int CHARACTER_INDEX = 0;
    private static final int WORD_INDEX = 1;
    private static final int LINE_INDEX = 2;
    private static final int SENTENCE_INDEX = 3;
    private static final SoftReference<BreakIterator.BreakIteratorCache>[] iterCache;
    
    protected BreakIterator() {
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    public abstract int first();
    
    public abstract int last();
    
    public abstract int next(int p0);
    
    public abstract int next();
    
    public abstract int previous();
    
    public abstract int following(int p0);
    
    public int preceding(int n) {
        int n2;
        for (n2 = this.following(n); n2 >= n && n2 != -1; n2 = this.previous()) {}
        return n2;
    }
    
    public boolean isBoundary(int n) {
        if (n == 0) {
            return true;
        }
        int following = this.following(n - 1);
        if (following == -1) {
            throw new IllegalArgumentException();
        }
        return following == n;
    }
    
    public abstract int current();
    
    public abstract CharacterIterator getText();
    
    public void setText(String s) {
        this.setText(new StringCharacterIterator(s));
    }
    
    public abstract void setText(CharacterIterator p0);
    
    public static BreakIterator getWordInstance() {
        return getWordInstance(Locale.getDefault());
    }
    
    public static BreakIterator getWordInstance(Locale locale) {
        return getBreakInstance(locale, 1);
    }
    
    public static BreakIterator getLineInstance() {
        return getLineInstance(Locale.getDefault());
    }
    
    public static BreakIterator getLineInstance(Locale locale) {
        return getBreakInstance(locale, 2);
    }
    
    public static BreakIterator getCharacterInstance() {
        return getCharacterInstance(Locale.getDefault());
    }
    
    public static BreakIterator getCharacterInstance(Locale locale) {
        return getBreakInstance(locale, 0);
    }
    
    public static BreakIterator getSentenceInstance() {
        return getSentenceInstance(Locale.getDefault());
    }
    
    public static BreakIterator getSentenceInstance(Locale locale) {
        return getBreakInstance(locale, 3);
    }
    
    private static BreakIterator getBreakInstance(Locale obj, int n) {
        if (BreakIterator.iterCache[n] != null) {
            BreakIterator.BreakIteratorCache breakIteratorCache = (BreakIterator.BreakIteratorCache)BreakIterator.iterCache[n].get();
            if (breakIteratorCache != null && breakIteratorCache.getLocale().equals(obj)) {
                return breakIteratorCache.createBreakInstance();
            }
        }
        BreakIterator breakInstance = createBreakInstance(obj, n);
        BreakIterator.iterCache[n] = new SoftReference(new BreakIterator.BreakIteratorCache(obj, breakInstance));
        return breakInstance;
    }
    
    private static BreakIterator createBreakInstance(Locale locale, int n) {
        BreakIterator breakIterator = createBreakInstance(LocaleProviderAdapter.getAdapter(BreakIteratorProvider.class, locale), locale, n);
        if (breakIterator == null) {
            breakIterator = createBreakInstance(LocaleProviderAdapter.forJRE(), locale, n);
            return breakIterator;
        }
        return breakIterator;
    }
    
    private static BreakIterator createBreakInstance(LocaleProviderAdapter localeProviderAdapter, Locale locale, int n) {
        BreakIteratorProvider breakIteratorProvider = localeProviderAdapter.getBreakIteratorProvider();
        BreakIterator breakIterator = null;
        switch (n) {
            case 0: {
                breakIterator = breakIteratorProvider.getCharacterInstance(locale);
                break;
            }
            case 1: {
                breakIterator = breakIteratorProvider.getWordInstance(locale);
                break;
            }
            case 2: {
                breakIterator = breakIteratorProvider.getLineInstance(locale);
                break;
            }
            case 3: {
                breakIterator = breakIteratorProvider.getSentenceInstance(locale);
                break;
            }
        }
        return breakIterator;
    }
    
    public static synchronized Locale[] getAvailableLocales() {
        return LocaleServiceProviderPool.getPool(BreakIteratorProvider.class).getAvailableLocales();
    }
    
    static {
        iterCache = (SoftReference[])new SoftReference[4];
    }
    
    private static final class BreakIteratorCache
    {
        private BreakIterator iter;
        private Locale locale;
        
        BreakIteratorCache(Locale locale, BreakIterator breakIterator) {
            this.locale = locale;
            this.iter = (BreakIterator)breakIterator.clone();
        }
        
        Locale getLocale() {
            return this.locale;
        }
        
        BreakIterator createBreakInstance() {
            return (BreakIterator)this.iter.clone();
        }
    }
}
